Plunk = LibStub("AceAddon-3.0"):NewAddon("Plunk","AceConsole-3.0","AceEvent-3.0","AceHook-3.0")

Plunk.enabled = false

local defaults = {
  profile = {
    names = {},
    vicinityAchievements = false,
    guildAchievements = false,
    canSpeak = false,
  },
}

local options = {
  name = "Plunk",
  handler = Plunk,
  type = 'group',
  args = {
    vicinityAchievements = {
      type = 'toggle',
      name = 'Achievements',
      desc = 'Ignore achievement spam in vicinity?',
      get = function (_) return Plunk.db.profile.vicinityAchievements end,
      set = function (_,v) Plunk.db.profile.vicinityAchievements = v end,
    },
    guildAchievements = {
      type = 'toggle',
      name = 'Guild Achievements',
      desc = 'Ignore achievement spam in guild?',
      get = function (_) return Plunk.db.profile.guildAchievements end,
      set = function (_,v) Plunk.db.profile.guildAchievements = v end,
    },
    canSpeak = {
      type = 'toggle',
      name = 'Other Languages',
      desc = 'Ignore messages in unintelligible languages?',
      get = function (_) return Plunk.db.profile.canSpeak end,
      set = function (_,v) Plunk.db.profile.canSpeak = v end,
    },
  },
}

function Plunk:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("PlunkDB", defaults, "Default")
  self:RegisterChatCommand("plunk", "PlunkChatCommand")
  self:RegisterChatCommand("unplunk", "UnplunkChatCommand")

  -- Setup Interface Options GUI
  LibStub("AceConfig-3.0"):RegisterOptionsTable("Plunk", options)
  self.optionsFrame = 
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Plunk", "Plunk")

  self:SetupFilters()
end

function Plunk:SetupFilters ()
  ChatFrame_AddMessageEventFilter("CHAT_MSG_ACHIEVEMENT",
    function (event)
      if Plunk.enabled then
        return self.db.profile.vicinityAchievements
      end
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD_ACHIEVEMENT",
    function (event)
      if Plunk.enabled then
        return self.db.profile.guildAchievements
      end
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_SAY",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_EMOTE",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_TEXT_EMOTE",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER",
    function (event)
      return self:FilterCallback(event, arg2, arg1, arg3)
    end)
end

-- Returns true if message is to be ignored
function Plunk:FilterCallback(event, name, text, lang)
  if Plunk.enabled then
    if self.db.profile.canSpeak and not self:CanSpeak(lang) then
      return true
    end
    return self:IsIgnored(name)
  end
end

function Plunk:OnEnable ()
  Plunk.enabled = true
end

function Plunk:OnDisable ()
  Plunk.enabled = false
end

function Plunk:PlunkChatCommand (input)
  if not input or input:trim() == "" then
    self:Print("Usage: /plunk <name>")
  else
    self.db.profile.names[input:trim():lower()] = true
  end
end

function Plunk:UnplunkChatCommand (input)
  if not input or input:trim() == "" then
    self:Print("Usage: /unplunk <name>")
  else
    self.db.profile.names[input:trim():lower()] = nil
  end
end

function Plunk:IsIgnored (name)
  if name then
    return self.db.profile.names[name:trim():lower()]
  end
end

function Plunk:CanSpeak (lang)
  if not lang or #lang == 0 then
    -- GMspeak may be classified as "nil"
    return true
  end
  for i=1, GetNumLanguages() do
    if GetLanguageByIndex(i) == lang then
      return true
    end
  end
  return false
end

